@@ -8,16 +8,17 @@ Demonstrates how to expand expressions.
 import sympy
 from sympy import pprint
 
+
 def main():
     a = sympy.Symbol('a')
     b = sympy.Symbol('b')
     e = (a + b)**5
 
-    print "\nExpression:"
+    print("\nExpression:")
     pprint(e)
-    print '\nExpansion of the above expression:'
+    print('\nExpansion of the above expression:')
     pprint(e.expand())
-    print
+    print()
 
 if __name__ == "__main__":
     main()
