@@ -17,7 +17,7 @@ fab vagrant func
 Even those functions that do not use vagrant must be run this way, because of
 the vagrant configuration at the bottom of this file.
 
-Any function that should be made avaiable from the command line needs to have
+Any function that should be made available from the command line needs to have
 the @task decorator.
 
 Save any files that should be reset between runs somewhere in the repos
@@ -136,7 +136,12 @@ def prepare_apt():
     dependencies.
     """
     sudo("apt-get -qq update")
-    sudo("apt-get -y install git python3 make python-virtualenv zip python-dev python-mpmath")
+    sudo("apt-get -y install git python3 make python-virtualenv zip python-dev python-mpmath python3-setuptools")
+    # Need 7.1.2 for Python 3.2 support
+    sudo("easy_install3 pip==7.1.2")
+    sudo("pip3 install mpmath")
+    # Be sure to use the Python 2 pip
+    sudo("/usr/bin/pip install twine")
     # Needed to build the docs
     sudo("apt-get -y install graphviz inkscape texlive texlive-xetex texlive-fonts-recommended texlive-latex-extra librsvg2-bin docbook2x")
     # Our Ubuntu is too old to include Python 3.3
@@ -303,7 +308,7 @@ def build_docs():
         with virtualenv(venv):
             with cd("/home/vagrant/repos/sympy/doc"):
                 run("make clean")
-                run("make html-errors")
+                run("make html")
                 run("make man")
                 with cd("/home/vagrant/repos/sympy/doc/_build"):
                     run("mv html {html-nozip}".format(**tarball_formatter()))
@@ -539,6 +544,8 @@ def table():
 
     table = []
 
+    version = get_sympy_version()
+
     # http://docs.python.org/2/library/contextlib.html#contextlib.contextmanager. Not
     # recommended as a real way to generate html, but it works better than
     # anything else I've tried.
@@ -547,6 +554,11 @@ def table():
         table.append("<%s>" % name)
         yield
         table.append("</%s>" % name)
+    @contextmanager
+    def a_href(link):
+        table.append("<a href=\"%s\">" % link)
+        yield
+        table.append("</a>")
 
     with tag('table'):
         with tag('tr'):
@@ -558,8 +570,9 @@ def table():
             name = get_tarball_name(key)
             with tag('tr'):
                 with tag('td'):
-                    with tag('b'):
-                        table.append(name)
+                    with a_href('https://github.com/sympy/sympy/releases/download/sympy-%s/%s' %(version,name)):
+                        with tag('b'):
+                            table.append(name)
                 with tag('td'):
                     table.append(descriptions[key].format(**tarball_formatter_dict))
                 with tag('td'):
@@ -927,8 +940,8 @@ def pypi_upload():
     Upload files to PyPI. You will need to enter a password.
     """
     with cd("/home/vagrant/repos/sympy"):
-        # See http://stackoverflow.com/a/17657183/161801
-        run("python setup.py sdist --dry-run upload")
+        run("twine upload dist/*.tar.gz")
+        run("twine upload dist/*.exe")
 
 @task
 def test_pypi(release='2'):
