@@ -1,17 +1,14 @@
 from __future__ import print_function, division
 
-from sympy.core.containers import Tuple
-from sympy.core.core import C
-from sympy.core.expr import Expr
+from sympy.tensor.indexed import Idx
 from sympy.core.mul import Mul
 from sympy.core.singleton import S
-from sympy.core.sympify import sympify
+from sympy.core.symbol import symbols
 from sympy.concrete.expr_with_intlimits import ExprWithIntLimits
-from sympy.functions.elementary.piecewise import piecewise_fold
 from sympy.functions.elementary.exponential import exp, log
 from sympy.polys import quo, roots
 from sympy.simplify import powsimp
-from sympy.core.compatibility import xrange
+from sympy.core.compatibility import range
 
 
 class Product(ExprWithIntLimits):
@@ -71,14 +68,14 @@ class Product(ExprWithIntLimits):
 
     >>> from sympy.abc import a, b, i, k, m, n, x
     >>> from sympy import Product, factorial, oo
-    >>> Product(k,(k,1,m))
+    >>> Product(k, (k, 1, m))
     Product(k, (k, 1, m))
-    >>> Product(k,(k,1,m)).doit()
+    >>> Product(k, (k, 1, m)).doit()
     factorial(m)
-    >>> Product(k**2,(k,1,m))
+    >>> Product(k**2,(k, 1, m))
     Product(k**2, (k, 1, m))
-    >>> Product(k**2,(k,1,m)).doit()
-    (factorial(m))**2
+    >>> Product(k**2,(k, 1, m)).doit()
+    factorial(m)**2
 
     Wallis' product for pi:
 
@@ -89,7 +86,7 @@ class Product(ExprWithIntLimits):
     Direct computation currently fails:
 
     >>> W.doit()
-    nan
+    Product(4*i**2/((2*i - 1)*(2*i + 1)), (i, 1, oo))
 
     But we can approach the infinite product by a limit of finite products:
 
@@ -99,20 +96,20 @@ class Product(ExprWithIntLimits):
     Product(4*i**2/((2*i - 1)*(2*i + 1)), (i, 1, n))
     >>> W2e = W2.doit()
     >>> W2e
-    2**(-2*n)*4**n*(factorial(n))**2/(RisingFactorial(1/2, n)*RisingFactorial(3/2, n))
+    2**(-2*n)*4**n*factorial(n)**2/(RisingFactorial(1/2, n)*RisingFactorial(3/2, n))
     >>> limit(W2e, n, oo)
     pi/2
 
     By the same formula we can compute sin(pi/2):
 
     >>> from sympy import pi, gamma, simplify
-    >>> P = pi * x * Product(1 - x**2/k**2,(k,1,n))
+    >>> P = pi * x * Product(1 - x**2/k**2, (k, 1, n))
     >>> P = P.subs(x, pi/2)
     >>> P
     pi**2*Product(1 - pi**2/(4*k**2), (k, 1, n))/2
     >>> Pe = P.doit()
     >>> Pe
-    pi**2*RisingFactorial(1 + pi/2, n)*RisingFactorial(-pi/2 + 1, n)/(2*(factorial(n))**2)
+    pi**2*RisingFactorial(1 + pi/2, n)*RisingFactorial(-pi/2 + 1, n)/(2*factorial(n)**2)
     >>> Pe = Pe.rewrite(gamma)
     >>> Pe
     pi**2*gamma(n + 1 + pi/2)*gamma(n - pi/2 + 1)/(2*gamma(1 + pi/2)*gamma(-pi/2 + 1)*gamma(n + 1)**2)
@@ -210,7 +207,6 @@ class Product(ExprWithIntLimits):
 
     def doit(self, **hints):
         f = self.function
-
         for index, limit in enumerate(self.limits):
             i, a, b = limit
             dif = b - a
@@ -219,7 +215,7 @@ class Product(ExprWithIntLimits):
                 f = 1 / f
 
             g = self._eval_product(f, (i, a, b))
-            if g is None:
+            if g in (None, S.NaN):
                 return self.func(powsimp(f), *self.limits[index:])
             else:
                 f = g
@@ -240,11 +236,13 @@ class Product(ExprWithIntLimits):
     def _eval_product(self, term, limits):
         from sympy.concrete.delta import deltaproduct, _has_simple_delta
         from sympy.concrete.summations import summation
-        from sympy.functions import KroneckerDelta
+        from sympy.functions import KroneckerDelta, RisingFactorial
 
         (k, a, n) = limits
 
         if k not in term.free_symbols:
+            if (term - 1).is_zero:
+                return S.One
             return term**(n - a + 1)
 
         if a == n:
@@ -255,7 +253,7 @@ class Product(ExprWithIntLimits):
 
         dif = n - a
         if dif.is_Integer:
-            return Mul(*[term.subs(k, a + i) for i in xrange(dif + 1)])
+            return Mul(*[term.subs(k, a + i) for i in range(dif + 1)])
 
         elif term.is_polynomial(k):
             poly = term.as_poly(k)
@@ -267,7 +265,7 @@ class Product(ExprWithIntLimits):
             M = 0
             for r, m in all_roots.items():
                 M += m
-                A *= C.RisingFactorial(a - r, n - a + 1)**m
+                A *= RisingFactorial(a - r, n - a + 1)**m
                 Q *= (n - r)**m
 
             if M < poly.degree():
@@ -278,10 +276,16 @@ class Product(ExprWithIntLimits):
 
         elif term.is_Add:
             p, q = term.as_numer_denom()
-
-            p = self._eval_product(p, (k, a, n))
             q = self._eval_product(q, (k, a, n))
+            if q.is_Number:
 
+                # There is expression, which couldn't change by
+                # as_numer_denom(). E.g. n**(2/3) + 1 --> (n**(2/3) + 1, 1).
+                # We have to catch this case.
+
+                p = sum([self._eval_product(i, (k, a, n)) for i in p.as_coeff_Add()])
+            else:
+                p = self._eval_product(p, (k, a, n))
             return p / q
 
         elif term.is_Mul:
@@ -331,6 +335,65 @@ class Product(ExprWithIntLimits):
             return self.func(self.function.transpose(), *self.limits)
         return None
 
+    def is_convergent(self):
+        r"""
+        See docs of Sum.is_convergent() for explanation of convergence
+        in SymPy.
+
+        The infinite product:
+
+        .. math::
+
+            \prod_{1 \leq i < \infty} f(i)
+
+        is defined by the sequence of partial products:
+
+        .. math::
+
+            \prod_{i=1}^{n} f(i) = f(1) f(2) \cdots f(n)
+
+        as n increases without bound. The product converges to a non-zero
+        value if and only if the sum:
+
+        .. math::
+
+            \sum_{1 \leq i < \infty} \log{f(n)}
+
+        converges.
+
+        References
+        ==========
+
+        .. [1] https://en.wikipedia.org/wiki/Infinite_product
+
+        Examples
+        ========
+
+        >>> from sympy import Interval, S, Product, Symbol, cos, pi, exp, oo
+        >>> n = Symbol('n', integer=True)
+        >>> Product(n/(n + 1), (n, 1, oo)).is_convergent()
+        False
+        >>> Product(1/n**2, (n, 1, oo)).is_convergent()
+        False
+        >>> Product(cos(pi/n), (n, 1, oo)).is_convergent()
+        True
+        >>> Product(exp(-n**2), (n, 1, oo)).is_convergent()
+        False
+        """
+        from sympy.concrete.summations import Sum
+
+        sequence_term = self.function
+        log_sum = log(sequence_term)
+        lim = self.limits
+        try:
+            is_conv = Sum(log_sum, *lim).is_convergent()
+        except NotImplementedError:
+            if Sum(sequence_term - 1, *lim).is_absolutely_convergent() is S.true:
+                return S.true
+            raise NotImplementedError("The algorithm to find the product convergence of %s "
+                                        "is not yet implemented" % (sequence_term))
+        return is_conv
+
     def reverse_order(expr, *indices):
         """
         Reverse the order of a limit in a Product.
@@ -371,18 +434,18 @@ class Product(ExprWithIntLimits):
         >>> S = Sum(x*y, (x, a, b), (y, c, d))
         >>> S
         Sum(x*y, (x, a, b), (y, c, d))
-        >>> S0 = S.reverse_order( 0)
+        >>> S0 = S.reverse_order(0)
         >>> S0
         Sum(-x*y, (x, b + 1, a - 1), (y, c, d))
-        >>> S1 = S0.reverse_order( 1)
+        >>> S1 = S0.reverse_order(1)
         >>> S1
         Sum(x*y, (x, b + 1, a - 1), (y, d + 1, c - 1))
 
         Of course we can mix both notations:
 
-        >>> Sum(x*y, (x, a, b), (y, 2, 5)).reverse_order( x, 1)
+        >>> Sum(x*y, (x, a, b), (y, 2, 5)).reverse_order(x, 1)
         Sum(x*y, (x, b + 1, a - 1), (y, 6, 1))
-        >>> Sum(x*y, (x, a, b), (y, 2, 5)).reverse_order( y, x)
+        >>> Sum(x*y, (x, a, b), (y, 2, 5)).reverse_order(y, x)
         Sum(x*y, (x, b + 1, a - 1), (y, 6, 1))
 
         See Also
@@ -409,7 +472,7 @@ class Product(ExprWithIntLimits):
             l = limit
             if i in l_indices:
                 e = -e
-                l = (limit[0], limit[2] + 1 , limit[1] - 1)
+                l = (limit[0], limit[2] + 1, limit[1] - 1)
             limits.append(l)
 
         return Product(expr.function ** e, *limits)
@@ -419,7 +482,7 @@ def product(*args, **kwargs):
     r"""
     Compute the product.
 
-    The notation for symbols is similiar to the notation used in Sum or
+    The notation for symbols is similar to the notation used in Sum or
     Integral. product(f, (i, a, b)) computes the product of f with
     respect to i from a to b, i.e.,
 
