@@ -1,15 +1,18 @@
 """This tests sympy/core/basic.py with (ideally) no reference to subclasses
 of Basic or Atom."""
 
+import collections
+import sys
+
 from sympy.core.basic import Basic, Atom, preorder_traversal
 from sympy.core.singleton import S, Singleton
 from sympy.core.symbol import symbols
 from sympy.core.compatibility import default_sort_key, with_metaclass
 
-from sympy import sin, Lambda, Q
-
+from sympy import sin, Lambda, Q, cos, gamma
+from sympy.functions.elementary.exponential import exp
 from sympy.utilities.pytest import raises
-
+from sympy.core import I, pi
 
 b1 = Basic()
 b2 = Basic(b1)
@@ -64,6 +67,10 @@ def test_subs():
     assert b21.subs([(b2, b1), (b1, b2)]) == Basic(b2, b2)
 
     assert b21.subs({b1: b2, b2: b1}) == Basic(b2, b2)
+    if sys.version_info >= (3, 3):
+        assert b21.subs(collections.ChainMap({b1: b2}, {b2: b1})) == Basic(b2, b2)
+    if sys.version_info >= (2, 7):
+        assert b21.subs(collections.OrderedDict([(b2, b1), (b1, b2)])) == Basic(b2, b2)
 
     raises(ValueError, lambda: b21.subs('bad arg'))
     raises(ValueError, lambda: b21.subs(b1, b2, b3))
@@ -107,6 +114,8 @@ def test_Singleton():
             instantiated += 1
             return Basic.__new__(cls)
 
+    assert instantiated == 0
+    MySingleton() # force instantiation
     assert instantiated == 1
     assert MySingleton() is not Basic()
     assert MySingleton() is MySingleton()
@@ -115,6 +124,8 @@ def test_Singleton():
 
     class MySingleton_sub(MySingleton):
         pass
+    assert instantiated == 1
+    MySingleton_sub()
     assert instantiated == 2
     assert MySingleton_sub() is not MySingleton()
     assert MySingleton_sub() is MySingleton_sub()
@@ -169,10 +180,21 @@ def test_call():
 
     assert (Q.real & Q.positive).rcall(x) == Q.real(x) & Q.positive(x)
 
+
+def test_rewrite():
+    x, y, z = symbols('x y z')
+    f1 = sin(x) + cos(x)
+    assert f1.rewrite(cos,exp) == exp(I*x)/2 + sin(x) + exp(-I*x)/2
+    assert f1.rewrite([cos],sin) == sin(x) + sin(x + pi/2, evaluate=False)
+    f2 = sin(x) + cos(y)/gamma(z)
+    assert f2.rewrite(sin,exp) == -I*(exp(I*x) - exp(-I*x))/2 + cos(y)/gamma(z)
+
+
 def test_literal_evalf_is_number_is_zero_is_comparable():
     from sympy.integrals.integrals import Integral
     from sympy.core.symbol import symbols
     from sympy.core.function import Function
+    from sympy.functions.elementary.trigonometric import cos, sin
     x = symbols('x')
     f = Function('f')
 
@@ -191,3 +213,9 @@ def test_literal_evalf_is_number_is_zero_is_comparable():
     assert i.is_zero
     assert i.is_number is False
     assert i.evalf(2, strict=False) == 0
+
+    # issue 10268
+    n = sin(1)**2 + cos(1)**2 - 1
+    assert n.is_comparable is False
+    assert n.n(2).is_comparable is False
+    assert n.n(2).n(2).is_comparable
