@@ -2,6 +2,7 @@ from sympy import Integer, S
 from sympy.core.operations import LatticeOp
 from sympy.utilities.pytest import raises
 from sympy.core.sympify import SympifyError
+from sympy.core.add import Add
 
 # create the simplest possible Lattice class
 
@@ -34,6 +35,7 @@ def test_lattice_print():
 
 
 def test_lattice_make_args():
-    assert join.make_args(0) == set([0])
-    assert join.make_args(1) == set([1])
-    assert join.make_args(join(2, 3, 4)) == set([S(2), S(3), S(4)])
+    assert join.make_args(join(2, 3, 4)) == {S(2), S(3), S(4)}
+    assert join.make_args(0) == {0}
+    assert list(join.make_args(0))[0] is S.Zero
+    assert Add.make_args(0)[0] is S.Zero
