@@ -1,8 +1,8 @@
 from __future__ import print_function, division
 
-from sympy.core import S, sympify, expand
+from sympy.core import S, sympify
+from sympy.core.compatibility import range
 from sympy.functions import Piecewise, piecewise_fold
-from sympy.functions.elementary.piecewise import ExprCondPair
 from sympy.sets.sets import Interval
 
 
@@ -43,7 +43,7 @@ def bspline_basis(d, knots, n, x, close=True):
         >>> d = 0
         >>> knots = range(5)
         >>> bspline_basis(d, knots, 0, x)
-        Piecewise((1, And(x <= 1, x >= 0)), (0, True))
+        Piecewise((1, (x >= 0) & (x <= 1)), (0, True))
 
     For a given ``(d, knots)`` there are ``len(knots)-d-1`` B-splines defined, that
     are indexed by ``n`` (starting at 0).
@@ -51,10 +51,13 @@ def bspline_basis(d, knots, n, x, close=True):
     Here is an example of a cubic B-spline:
 
         >>> bspline_basis(3, range(5), 0, x)
-        Piecewise((x**3/6, And(x < 1, x >= 0)),
-                  (-x**3/2 + 2*x**2 - 2*x + 2/3, And(x < 2, x >= 1)),
-                  (x**3/2 - 4*x**2 + 10*x - 22/3, And(x < 3, x >= 2)),
-                  (-x**3/6 + 2*x**2 - 8*x + 32/3, And(x <= 4, x >= 3)),
+        Piecewise((x**3/6, (x >= 0) & (x < 1)),
+                  (-x**3/2 + 2*x**2 - 2*x + 2/3,
+                  (x >= 1) & (x < 2)),
+                  (x**3/2 - 4*x**2 + 10*x - 22/3,
+                  (x >= 2) & (x < 3)),
+                  (-x**3/6 + 2*x**2 - 8*x + 32/3,
+                  (x >= 3) & (x <= 4)),
                   (0, True))
 
     By repeating knot points, you can introduce discontinuities in the
@@ -63,7 +66,7 @@ def bspline_basis(d, knots, n, x, close=True):
         >>> d = 1
         >>> knots = [0,0,2,3,4]
         >>> bspline_basis(d, knots, 0, x)
-        Piecewise((-x/2 + 1, And(x <= 2, x >= 0)), (0, True))
+        Piecewise((-x/2 + 1, (x >= 0) & (x <= 2)), (0, True))
 
     It is quite time consuming to construct and evaluate B-splines. If you
     need to evaluate a B-splines many times, it is best to lambdify them
@@ -138,14 +141,14 @@ def bspline_basis_set(d, knots, x):
     >>> knots = range(5)
     >>> splines = bspline_basis_set(d, knots, x)
     >>> splines
-    [Piecewise((x**2/2, And(x < 1, x >= 0)),
-               (-x**2 + 3*x - 3/2, And(x < 2, x >= 1)),
-               (x**2/2 - 3*x + 9/2, And(x <= 3, x >= 2)),
+    [Piecewise((x**2/2, (x >= 0) & (x < 1)),
+               (-x**2 + 3*x - 3/2, (x >= 1) & (x < 2)),
+               (x**2/2 - 3*x + 9/2, (x >= 2) & (x <= 3)),
                (0, True)),
-     Piecewise((x**2/2 - x + 1/2, And(x < 2, x >= 1)),
-               (-x**2 + 5*x - 11/2, And(x < 3, x >= 2)),
-               (x**2/2 - 4*x + 8, And(x <= 4, x >= 3)),
-               (0, True))]
+    Piecewise((x**2/2 - x + 1/2, (x >= 1) & (x < 2)),
+              (-x**2 + 5*x - 11/2, (x >= 2) & (x < 3)),
+              (x**2/2 - 4*x + 8, (x >= 3) & (x <= 4)),
+              (0, True))]
 
     See Also
     ========
