@@ -28,15 +28,20 @@ The main references for this are:
 from __future__ import print_function, division
 
 from sympy.core import oo, S, pi, Expr
+from sympy.core.exprtools import factor_terms
 from sympy.core.function import expand, expand_mul, expand_power_base
 from sympy.core.add import Add
 from sympy.core.mul import Mul
+from sympy.core.compatibility import range
 from sympy.core.cache import cacheit
 from sympy.core.symbol import Dummy, Wild
-from sympy.simplify import hyperexpand, powdenest
+from sympy.simplify import hyperexpand, powdenest, collect
 from sympy.logic.boolalg import And, Or, BooleanAtom
 from sympy.functions.special.delta_functions import Heaviside
-from sympy.functions.elementary.piecewise import Piecewise
+from sympy.functions.elementary.exponential import exp
+from sympy.functions.elementary.piecewise import Piecewise, piecewise_fold
+from sympy.functions.elementary.hyperbolic import \
+    _rewrite_hyperbolics_as_exp, HyperbolicFunction
 from sympy.functions.special.hyper import meijerg
 from sympy.utilities.iterables import multiset_partitions, ordered
 from sympy.utilities.misc import debug as _debug
@@ -46,6 +51,15 @@ from sympy.utilities import default_sort_key
 z = Dummy('z')
 
 
+def _has(res, *f):
+    # return True if res has f; in the case of Piecewise
+    # only return True if *all* pieces have f
+    res = piecewise_fold(res)
+    if getattr(res, 'is_Piecewise', False):
+        return all(_has(i, *f) for i in res.args)
+    return res.has(*f)
+
+
 def _create_lookup_table(table):
     """ Add formulae for the function -> meijerg lookup table. """
     def wild(n):
@@ -80,7 +94,7 @@ def _create_lookup_table(table):
                 return arg <= 0
 
     # Section 8.4.2
-    from sympy import (gamma, pi, cos, exp, re, sin, sqrt, sinh, cosh,
+    from sympy import (gamma, pi, cos, exp, re, sin, sinc, sqrt, sinh, cosh,
                        factorial, log, erf, erfc, erfi, polar_lift)
     # TODO this needs more polar_lift (c/f entry for exp)
     add(Heaviside(t - b)*(t - b)**(a - 1), [a], [], [], [0], t/b,
@@ -94,7 +108,7 @@ def _create_lookup_table(table):
     add((b + t)**(-a), [1 - a], [], [0], [], t/b, b**(-a)/gamma(a),
         hint=Not(IsNonPositiveInteger(a)))
     add(abs(b - t)**(-a), [1 - a], [(1 - a)/2], [0], [(1 - a)/2], t/b,
-        pi/(gamma(a)*cos(pi*a/2))*abs(b)**(-a), re(a) < 1)
+        2*sin(pi*a/2)*gamma(1 - a)*abs(b)**(-a), re(a) < 1)
     add((t**a - b**a)/(t - b), [0, a], [], [0, a], [], t/b,
         b**(a - 1)*sin(a*pi)/pi)
 
@@ -137,6 +151,9 @@ def _create_lookup_table(table):
     add(sin(t), [], [], [S(1)/2], [0], t**2/4, sqrt(pi))
     add(cos(t), [], [], [0], [S(1)/2], t**2/4, sqrt(pi))
 
+    # Section 8.4.6 (sinc function)
+    add(sinc(t), [], [], [0], [S(-1)/2], t**2/4, sqrt(pi)/2)
+
     # Section 8.5.5
     def make_log1(subs):
         N = subs[n]
@@ -326,13 +343,13 @@ def _exponents(expr, x):
     >>> from sympy.abc import x, y
     >>> from sympy import sin
     >>> _exponents(x, x)
-    set([1])
+    {1}
     >>> _exponents(x**2, x)
-    set([2])
+    {2}
     >>> _exponents(x**2 + x, x)
-    set([1, 2])
+    {1, 2}
     >>> _exponents(x**3*sin(x + x**y) + 1/x, x)
-    set([-1, 1, 3, y])
+    {-1, 1, 3, y}
     """
     def _exponents_(expr, x, res):
         if expr == x:
@@ -356,20 +373,19 @@ def _functions(expr, x):
 
 def _find_splitting_points(expr, x):
     """
-    Find numbers a such that a linear substitution x --> x+a would
+    Find numbers a such that a linear substitution x -> x + a would
     (hopefully) simplify expr.
 
     >>> from sympy.integrals.meijerint import _find_splitting_points as fsp
     >>> from sympy import sin
     >>> from sympy.abc import a, x
     >>> fsp(x, x)
-    set([0])
+    {0}
     >>> fsp((x-1)**3, x)
-    set([1])
+    {1}
     >>> fsp(sin(x+3)*x, x)
-    set([-3, 0])
+    {-3, 0}
     """
-    from sympy import Tuple
     p, q = [Wild(n, exclude=[x]) for n in 'pq']
 
     def compute_innermost(expr, res):
@@ -458,9 +474,6 @@ def _mul_as_two_parts(f):
     if the case of len(gs) == 2 is removed and multiset is allowed to
     sort the terms, some tests fail.
 
-    Examples
-    ========
-
     >>> from sympy.integrals.meijerint import _mul_as_two_parts
     >>> from sympy import sin, exp, ordered
     >>> from sympy.abc import x
@@ -574,7 +587,7 @@ def _condsimp(cond):
     >>> from sympy import Or, Eq, unbranched_argument as arg, And
     >>> from sympy.abc import x, y, z
     >>> simp(Or(x < y, z, Eq(x, y)))
-    Or(x <= y, z)
+    z | (x <= y)
     >>> simp(Or(x <= y, And(x < y, z)))
     x <= y
     """
@@ -704,7 +717,7 @@ def _rewrite_saxena_1(fac, po, g, x):
 
 
 def _check_antecedents_1(g, x, helper=False):
-    """
+    r"""
     Return a condition under which the mellin transform of g exists.
     Any power of x has already been absorbed into the G function,
     so this is just int_0^\infty g dx.
@@ -793,7 +806,7 @@ def _check_antecedents_1(g, x, helper=False):
     # TODO altered cases 4-7
 
     # extra case from wofram functions site:
-    # (reproduced verbatim from prudnikov, section 2.24.2)
+    # (reproduced verbatim from Prudnikov, section 2.24.2)
     # http://functions.wolfram.com/HypergeometricFunctions/MeijerG/21/02/01/
     case_extra = []
     case_extra += [Eq(p, q), Eq(delta, 0), Eq(arg(eta), 0), Ne(eta, 0)]
@@ -814,14 +827,14 @@ def _check_antecedents_1(g, x, helper=False):
     conds += [case_extra_2]
     debug('  second extra case:', [case_extra_2])
 
-    # TODO This leaves only one case from the three listed by prudnikov.
+    # TODO This leaves only one case from the three listed by Prudnikov.
     #      Investigate if these indeed cover everything; if so, remove the rest.
 
     return Or(*conds)
 
 
 def _int0oo_1(g, x):
-    """
+    r"""
     Evaluate int_0^\infty g dx using G functions,
     assuming the necessary conditions are fulfilled.
 
@@ -916,8 +929,7 @@ def _rewrite_saxena(fac, po, g1, g2, x, full_pb=False):
 
 def _check_antecedents(g1, g2, x):
     """ Return a condition under which the integral theorem applies. """
-    from sympy import (re, Eq, Not, Ne, cos, I, exp, ceiling, sin, sign,
-                       unpolarify)
+    from sympy import re, Eq, Ne, cos, I, exp, sin, sign, unpolarify
     from sympy import arg as arg_, unbranched_argument as arg
     #  Yes, this is madness.
     # XXX TODO this is a testing *nightmare*
@@ -942,20 +954,6 @@ def _check_antecedents(g1, g2, x):
     eta = 1 - (v - u) - mu - rho
     psi = (pi*(q - m - n) + abs(arg(omega)))/(q - p)
     theta = (pi*(v - s - t) + abs(arg(sigma)))/(v - u)
-    lambda_c = (q - p)*abs(omega)**(1/(q - p))*cos(psi) \
-        + (v - u)*abs(sigma)**(1/(v - u))*cos(theta)
-
-    def lambda_s0(c1, c2):
-        return c1*(q - p)*abs(omega)**(1/(q - p))*sin(psi) \
-            + c2*(v - u)*abs(sigma)**(1/(v - u))*sin(theta)
-    lambda_s = Piecewise(
-        ((lambda_s0(+1, +1)*lambda_s0(-1, -1)),
-         And(Eq(arg(sigma), 0), Eq(arg(omega), 0))),
-        (lambda_s0(sign(arg(omega)), +1)*lambda_s0(sign(arg(omega)), -1),
-         And(Eq(arg(sigma), 0), Ne(arg(omega), 0))),
-        (lambda_s0(+1, sign(arg(sigma)))*lambda_s0(-1, sign(arg(sigma))),
-         And(Ne(arg(sigma), 0), Eq(arg(omega), 0))),
-        (lambda_s0(sign(arg(omega)), sign(arg(sigma))), True))
 
     _debug('Checking antecedents:')
     _debug('  sigma=%s, s=%s, t=%s, u=%s, v=%s, b*=%s, rho=%s'
@@ -964,46 +962,21 @@ def _check_antecedents(g1, g2, x):
            % (omega, m, n, p, q, cstar, mu))
     _debug('  phi=%s, eta=%s, psi=%s, theta=%s' % (phi, eta, psi, theta))
 
-    c1 = True
-    for g in [g1, g2]:
-        for a in g1.an:
-            for b in g1.bm:
-                diff = a - b
-                if (diff > 0) == True and diff.is_integer:
-                    c1 = False
-
-    tmp = []
-    for b in g1.bm:
-        for d in g2.bm:
-            tmp += [re(1 + b + d) > 0]
-    c2 = And(*tmp)
-
-    tmp = []
-    for a in g1.an:
-        for c in g2.an:
-            tmp += [re(1 + a + c) < 1 + 1]
-    c3 = And(*tmp)
-
-    tmp = []
-    for c in g1.an:
-        tmp += [(p - q)*re(1 + c - 1) - re(mu) > -S(3)/2]
-    c4 = And(*tmp)
-
-    tmp = []
-    for d in g1.bm:
-        tmp += [(p - q)*re(1 + d) - re(mu) > -S(3)/2]
-    c5 = And(*tmp)
-
-    tmp = []
-    for c in g2.an:
-        tmp += [(u - v)*re(1 + c - 1) - re(rho) > -S(3)/2]
-    c6 = And(*tmp)
-
-    tmp = []
-    for d in g2.bm:
-        tmp += [(u - v)*re(1 + d) - re(rho) > -S(3)/2]
-    c7 = And(*tmp)
-
+    def _c1():
+        for g in [g1, g2]:
+            for i in g.an:
+                for j in g.bm:
+                    diff = i - j
+                    if diff.is_integer and diff.is_positive:
+                        return False
+        return True
+    c1 = _c1()
+    c2 = And(*[re(1 + i + j) > 0 for i in g1.bm for j in g2.bm])
+    c3 = And(*[re(1 + i + j) < 1 + 1 for i in g1.an for j in g2.an])
+    c4 = And(*[(p - q)*re(1 + i - 1) - re(mu) > -S(3)/2 for i in g1.an])
+    c5 = And(*[(p - q)*re(1 + i) - re(mu) > -S(3)/2 for i in g1.bm])
+    c6 = And(*[(u - v)*re(1 + i - 1) - re(rho) > -S(3)/2 for i in g2.an])
+    c7 = And(*[(u - v)*re(1 + i) - re(rho) > -S(3)/2 for i in g2.bm])
     c8 = (abs(phi) + 2*re((rho - 1)*(q - p) + (v - u)*(q - p) + (mu -
           1)*(v - u)) > 0)
     c9 = (abs(phi) - 2*re((rho - 1)*(q - p) + (v - u)*(q - p) + (mu -
@@ -1014,7 +987,7 @@ def _check_antecedents(g1, g2, x):
     c13 = Eq(abs(arg(omega)), cstar*pi)
 
     # The following condition is *not* implemented as stated on the wolfram
-    # function site. In the book of prudnikov there is an additional part
+    # function site. In the book of Prudnikov there is an additional part
     # (the And involving re()). However, I only have this book in russian, and
     # I don't read any russian. The following condition is what other people
     # have told me it means.
@@ -1036,8 +1009,19 @@ def _check_antecedents(g1, g2, x):
                   Or(And(Ne(zos, 1), abs(arg_(1 - zos)) < pi),
                      And(re(mu + rho + v - u) < 1, Eq(zos, 1))))
 
+        def _cond():
+            '''
+            Note: if `zso` is 1 then tmp will be NaN.  This raises a
+            TypeError on `NaN < pi`.  Previously this gave `False` so
+            this behavior has been hardcoded here but someone should
+            check if this NaN is more serious! This NaN is triggered by
+            test_meijerint() in test_meijerint.py:
+            `meijerint_definite(exp(x), x, 0, I)`
+            '''
+            tmp = abs(arg_(1 - zso))
+            return False if tmp is S.NaN else tmp < pi
         c14_alt = And(Eq(phi, 0), cstar - 1 + bstar <= 0,
-                  Or(And(Ne(zso, 1), abs(arg_(1 - zso)) < pi),
+                  Or(And(Ne(zso, 1), _cond()),
                      And(re(mu + rho + q - p) < 1, Eq(zso, 1))))
 
         # Since r=k=l=1, in our case there is c14_alt which is the same as calling
@@ -1049,13 +1033,38 @@ def _check_antecedents(g1, g2, x):
         # Hence the following seems correct:
         c14 = Or(c14, c14_alt)
 
-    tmp = [lambda_c > 0,
-           And(Eq(lambda_c, 0), Ne(lambda_s, 0), re(eta) > -1),
-           And(Eq(lambda_c, 0), Eq(lambda_s, 0), re(eta) > 0)]
-    c15 = Or(*tmp)
-    if _eval_cond(lambda_c > 0) != False:
-        c15 = (lambda_c > 0)
-
+    '''
+    When `c15` is NaN (e.g. from `psi` being NaN as happens during
+    'test_issue_4992' and/or `theta` is NaN as in 'test_issue_6253',
+    both in `test_integrals.py`) the comparison to 0 formerly gave False
+    whereas now an error is raised. To keep the old behavior, the value
+    of NaN is replaced with False but perhaps a closer look at this condition
+    should be made: XXX how should conditions leading to c15=NaN be handled?
+    '''
+    try:
+        lambda_c = (q - p)*abs(omega)**(1/(q - p))*cos(psi) \
+            + (v - u)*abs(sigma)**(1/(v - u))*cos(theta)
+        # the TypeError might be raised here, e.g. if lambda_c is NaN
+        if _eval_cond(lambda_c > 0) != False:
+            c15 = (lambda_c > 0)
+        else:
+            def lambda_s0(c1, c2):
+                return c1*(q - p)*abs(omega)**(1/(q - p))*sin(psi) \
+                    + c2*(v - u)*abs(sigma)**(1/(v - u))*sin(theta)
+            lambda_s = Piecewise(
+                ((lambda_s0(+1, +1)*lambda_s0(-1, -1)),
+                 And(Eq(arg(sigma), 0), Eq(arg(omega), 0))),
+                (lambda_s0(sign(arg(omega)), +1)*lambda_s0(sign(arg(omega)), -1),
+                 And(Eq(arg(sigma), 0), Ne(arg(omega), 0))),
+                (lambda_s0(+1, sign(arg(sigma)))*lambda_s0(-1, sign(arg(sigma))),
+                 And(Ne(arg(sigma), 0), Eq(arg(omega), 0))),
+                (lambda_s0(sign(arg(omega)), sign(arg(sigma))), True))
+            tmp = [lambda_c > 0,
+                   And(Eq(lambda_c, 0), Ne(lambda_s, 0), re(eta) > -1),
+                   And(Eq(lambda_c, 0), Eq(lambda_s, 0), re(eta) > 0)]
+            c15 = Or(*tmp)
+    except TypeError:
+        c15 = False
     for cond, i in [(c1, 1), (c2, 2), (c3, 3), (c4, 4), (c5, 5), (c6, 6),
                     (c7, 7), (c8, 8), (c9, 9), (c10, 10), (c11, 11),
                     (c12, 12), (c13, 13), (c14, 14), (c15, 15)]:
@@ -1135,7 +1144,7 @@ def _check_antecedents(g1, g2, x):
     pr(23)
 
     # The following case is from [Luke1969]. As far as I can tell, it is *not*
-    # covered by prudnikov's.
+    # covered by Prudnikov's.
     # Let G1 and G2 be the two G-functions. Suppose the integral exists from
     # 0 to a > 0 (this is easy the easy part), that G1 is exponential decay at
     # infinity, and that the mellin transform of G2 exists.
@@ -1333,7 +1342,7 @@ def _check_antecedents_inversion(g, x):
     # When p < q, we need to use the theorems of [L], 5.10.
 
     if p >= q:
-        _debug('  Using asymptotic slater expansion.')
+        _debug('  Using asymptotic Slater expansion.')
         return And(*[statement(a - 1, 0, 0, z) for a in g.an])
 
     def E(z):
@@ -1452,7 +1461,10 @@ def _rewrite_single(f, x, recursive=True):
                 for fac, g in terms:
                     r1 = _get_coeff_exp(unpolarify(fac.subs(subs).subs(z, x),
                                                    exponents_only=True), x)
-                    g = g.subs(subs).subs(z, x)
+                    try:
+                        g = g.subs(subs).subs(z, x)
+                    except ValueError:
+                        continue
                     # NOTE these substitutions can in principle introduce oo,
                     #      zoo and other absurdities. It shouldn't matter,
                     #      but better be safe.
@@ -1467,7 +1479,7 @@ def _rewrite_single(f, x, recursive=True):
     # try recursive mellin transform
     if not recursive:
         return None
-    _debug('Trying recursive mellin transform method.')
+    _debug('Trying recursive Mellin transform method.')
     from sympy.integrals.transforms import (mellin_transform,
                                     inverse_mellin_transform, IntegralTransformError,
                                     MellinTransformStripError)
@@ -1518,7 +1530,7 @@ def _rewrite_single(f, x, recursive=True):
             except IntegralTransformError:
                 g = None
     if g is None or g.has(oo, nan, zoo):
-        _debug('Recursive mellin transform failed.')
+        _debug('Recursive Mellin transform failed.')
         return None
     args = Add.make_args(g)
     res = []
@@ -1532,7 +1544,7 @@ def _rewrite_single(f, x, recursive=True):
                                unpolarify(polarify(
                                    a, lift=True), exponents_only=True)
                                *x**b))]
-    _debug('Recursive mellin transform worked:', g)
+    _debug('Recursive Mellin transform worked:', g)
     return res, True
 
 
@@ -1584,28 +1596,42 @@ def meijerint_indefinite(f, x):
     """
     Compute an indefinite integral of ``f`` by rewriting it as a G function.
 
+    Examples
+    ========
+
     >>> from sympy.integrals.meijerint import meijerint_indefinite
     >>> from sympy import sin
     >>> from sympy.abc import x
     >>> meijerint_indefinite(sin(x), x)
     -cos(x)
     """
-    from sympy import hyper, meijerg, count_ops
+    from sympy import hyper, meijerg
+
     results = []
-    for a in sorted(_find_splitting_points(f, x) | set([S(0)]), key=default_sort_key):
+    for a in sorted(_find_splitting_points(f, x) | {S(0)}, key=default_sort_key):
         res = _meijerint_indefinite_1(f.subs(x, x + a), x)
-        if res is None:
+        if not res:
             continue
-        results.append(res.subs(x, x - a))
-        if not res.has(hyper, meijerg):
-            return results[-1]
+        res = res.subs(x, x - a)
+        if _has(res, hyper, meijerg):
+            results.append(res)
+        else:
+            return res
+    if f.has(HyperbolicFunction):
+        _debug('Try rewriting hyperbolics in terms of exp.')
+        rv = meijerint_indefinite(
+            _rewrite_hyperbolics_as_exp(f), x)
+        if rv:
+            if not type(rv) is list:
+                return collect(factor_terms(rv), rv.atoms(exp))
+            results.extend(rv)
     if results:
         return next(ordered(results))
 
 
 def _meijerint_indefinite_1(f, x):
     """ Helper that does not attempt any substitution. """
-    from sympy import Integral, piecewise_fold
+    from sympy import Integral, piecewise_fold, nan, zoo
     _debug('Trying to compute the indefinite integral of', f, 'wrt', x)
 
     gs = _rewrite1(f, x)
@@ -1643,7 +1669,12 @@ def _meijerint_indefinite_1(f, x):
         else:
             r = meijerg(
                 tr(g.an) + [1], tr(g.aother), tr(g.bm), tr(g.bother) + [0], t)
-        r = hyperexpand(r.subs(t, a*x**b))
+        # The antiderivative is most often expected to be defined
+        # in the neighborhood of  x = 0.
+        place = 0
+        if b < 0 or f.subs(x, 0).has(nan, zoo):
+            place = None
+        r = hyperexpand(r.subs(t, a*x**b), place=place)
 
         # now substitute back
         # Note: we really do want the powers of x to combine.
@@ -1694,6 +1725,9 @@ def meijerint_definite(f, x, a, b):
 
     Return res, cond, where cond are convergence conditions.
 
+    Examples
+    ========
+
     >>> from sympy.integrals.meijerint import meijerint_definite
     >>> from sympy import exp, oo
     >>> from sympy.abc import x
@@ -1713,13 +1747,17 @@ def meijerint_definite(f, x, a, b):
     #
     # There are usually several ways of doing this, and we want to try all.
     # This function does (1), calls _meijerint_definite_2 for step (2).
-    from sympy import Integral, arg, exp, I, And, DiracDelta, count_ops
+    from sympy import arg, exp, I, And, DiracDelta, SingularityFunction
     _debug('Integrating', f, 'wrt %s from %s to %s.' % (x, a, b))
 
     if f.has(DiracDelta):
         _debug('Integrand has DiracDelta terms - giving up.')
         return None
 
+    if f.has(SingularityFunction):
+        _debug('Integrand has Singularity Function terms - giving up.')
+        return None
+
     f_, x_, a_, b_ = f, x, a, b
 
     # Let's use a dummy in case any of the boundaries has x.
@@ -1727,13 +1765,14 @@ def meijerint_definite(f, x, a, b):
     f = f.subs(x, d)
     x = d
 
-    if (a == oo and b == oo) or (a == -oo and b == -oo):
-        return (0, True)
+    if a == b:
+        return (S.Zero, True)
 
+    results = []
     if a == -oo and b != oo:
         return meijerint_definite(f.subs(x, -x), x, -b, -a)
 
-    if a == -oo:
+    elif a == -oo:
         # Integrating -oo to oo. We need to find a place to split the integral.
         _debug('  Integrating -oo to +oo.')
         innermost = _find_splitting_points(f, x)
@@ -1759,71 +1798,86 @@ def meijerint_definite(f, x, a, b):
                 continue
             res = res1 + res2
             return res, cond
-        return
 
-    if a == oo:
+    elif a == oo:
         return -meijerint_definite(f, x, b, oo)
 
-    if (a, b) == (0, oo):
+    elif (a, b) == (0, oo):
         # This is a common case - try it directly first.
         res = _meijerint_definite_2(f, x)
-        if res is not None and not res[0].has(meijerg):
-            return res
+        if res:
+            if _has(res[0], meijerg):
+                results.append(res)
+            else:
+                return res
 
-    results = []
-    if b == oo:
-        for split in _find_splitting_points(f, x):
-            if (a - split >= 0) == True:
-                _debug('Trying x --> x + %s' % split)
-                res = _meijerint_definite_2(f.subs(x, x + split)
-                                            *Heaviside(x + split - a), x)
-                if res is not None:
-                    if res[0].has(meijerg):
-                        results.append(res)
-                    else:
-                        return res
+    else:
+        if b == oo:
+            for split in _find_splitting_points(f, x):
+                if (a - split >= 0) == True:
+                    _debug('Trying x -> x + %s' % split)
+                    res = _meijerint_definite_2(f.subs(x, x + split)
+                                                *Heaviside(x + split - a), x)
+                    if res:
+                        if _has(res[0], meijerg):
+                            results.append(res)
+                        else:
+                            return res
 
-    f = f.subs(x, x + a)
-    b = b - a
-    a = 0
-    if b != oo:
-        phi = exp(I*arg(b))
-        b = abs(b)
-        f = f.subs(x, phi*x)
-        f *= Heaviside(b - x)*phi
-        b = oo
+        f = f.subs(x, x + a)
+        b = b - a
+        a = 0
+        if b != oo:
+            phi = exp(I*arg(b))
+            b = abs(b)
+            f = f.subs(x, phi*x)
+            f *= Heaviside(b - x)*phi
+            b = oo
 
-    _debug('Changed limits to', a, b)
-    _debug('Changed function to', f)
-    res = _meijerint_definite_2(f, x)
-    if res is not None:
-        if res[0].has(meijerg):
-            results.append(res)
-        else:
-            return res
+        _debug('Changed limits to', a, b)
+        _debug('Changed function to', f)
+        res = _meijerint_definite_2(f, x)
+        if res:
+            if _has(res[0], meijerg):
+                results.append(res)
+            else:
+                return res
+    if f_.has(HyperbolicFunction):
+        _debug('Try rewriting hyperbolics in terms of exp.')
+        rv = meijerint_definite(
+            _rewrite_hyperbolics_as_exp(f_), x_, a_, b_)
+        if rv:
+            if not type(rv) is list:
+                rv = (collect(factor_terms(rv[0]), rv[0].atoms(exp)),) + rv[1:]
+                return rv
+            results.extend(rv)
     if results:
-        return sorted(results, key=lambda x: count_ops(x[0]))[0]
+        return next(ordered(results))
 
 
 def _guess_expansion(f, x):
     """ Try to guess sensible rewritings for integrand f(x). """
     from sympy import expand_trig
     from sympy.functions.elementary.trigonometric import TrigonometricFunction
-    from sympy.functions.elementary.hyperbolic import HyperbolicFunction
-    res = [(f, 'originial integrand')]
+    res = [(f, 'original integrand')]
 
-    expanded = expand_mul(res[-1][0])
-    if expanded != res[-1][0]:
+    orig = res[-1][0]
+    saw = {orig}
+    expanded = expand_mul(orig)
+    if expanded not in saw:
         res += [(expanded, 'expand_mul')]
+        saw.add(expanded)
 
-    expanded = expand(res[-1][0])
-    if expanded != res[-1][0]:
+    expanded = expand(orig)
+    if expanded not in saw:
         res += [(expanded, 'expand')]
+        saw.add(expanded)
 
-    if res[-1][0].has(TrigonometricFunction, HyperbolicFunction):
-        expanded = expand_mul(expand_trig(res[-1][0]))
-        if expanded != res[-1][0]:
+    if orig.has(TrigonometricFunction, HyperbolicFunction):
+        expanded = expand_mul(expand_trig(orig))
+        if expanded not in saw:
             res += [(expanded, 'expand_trig, expand_mul')]
+            saw.add(expanded)
 
     return res
 
@@ -1843,6 +1897,7 @@ def _meijerint_definite_2(f, x):
     # _meijerint_definite_3 for (2) and (3) combined.
 
     # use a positive dummy - we integrate from 0 to oo
+    # XXX if a nonnegative symbol is used there will be test failures
     dummy = _dummy('x', 'meijerint-definite2', f, positive=True)
     f = f.subs(x, dummy)
     x = dummy
@@ -1853,7 +1908,7 @@ def _meijerint_definite_2(f, x):
     for g, explanation in _guess_expansion(f, x):
         _debug('Trying', explanation)
         res = _meijerint_definite_3(g, x)
-        if res is not None and res[1] != False:
+        if res:
             return res
 
 
@@ -1865,7 +1920,7 @@ def _meijerint_definite_3(f, x):
     integral. If this fails, it tries using linearity.
     """
     res = _meijerint_definite_4(f, x)
-    if res is not None and res[1] != False:
+    if res and res[1] != False:
         return res
     if f.is_Add:
         _debug('Expanding and evaluating all terms.')
@@ -1912,11 +1967,13 @@ def _meijerint_definite_4(f, x, only_double=False):
                 C, f = _rewrite_saxena_1(fac*C, po*x**s, f, x)
                 res += C*_int0oo_1(f, x)
                 cond = And(cond, _check_antecedents_1(f, x))
+                if cond == False:
+                    break
             cond = _my_unpolarify(cond)
-            _debug('Result before branch substitutions is:', res)
             if cond == False:
                 _debug('But cond is always False.')
             else:
+                _debug('Result before branch substitutions is:', res)
                 return _my_unpolarify(hyperexpand(res)), cond
 
     # Try two G functions.
@@ -1936,19 +1993,24 @@ def _meijerint_definite_4(f, x, only_double=False):
                     C, f1_, f2_ = r
                     _debug('Saxena subst for yielded:', C, f1_, f2_)
                     cond = And(cond, _check_antecedents(f1_, f2_, x))
+                    if cond == False:
+                        break
                     res += C*_int0oo(f1_, f2_, x)
-            _debug('Result before branch substitutions is:', res)
+                else:
+                    continue
+                break
             cond = _my_unpolarify(cond)
             if cond == False:
                 _debug('But cond is always False (full_pb=%s).' % full_pb)
             else:
+                _debug('Result before branch substitutions is:', res)
                 if only_double:
                     return res, cond
                 return _my_unpolarify(hyperexpand(res)), cond
 
 
 def meijerint_inversion(f, x, t):
-    """
+    r"""
     Compute the inverse laplace transform
     :math:\int_{c+i\infty}^{c-i\infty} f(x) e^{tx) dx,
     for real c larger than the real part of all singularities of f.
@@ -1956,6 +2018,9 @@ def meijerint_inversion(f, x, t):
 
     Return None if the integral does not exist or could not be evaluated.
 
+    Examples
+    ========
+
     >>> from sympy.abc import x, t
     >>> from sympy.integrals.meijerint import meijerint_inversion
     >>> meijerint_inversion(1/x, x, t)
@@ -2020,6 +2085,8 @@ def meijerint_inversion(f, x, t):
             C, f = _rewrite_inversion(fac*C, po*x**s, f, x)
             res += C*_int_inversion(f, x, t)
             cond = And(cond, _check_antecedents_inversion(f, x))
+            if cond == False:
+                break
         cond = _my_unpolarify(cond)
         if cond == False:
             _debug('But cond is always False.')
