@@ -3,13 +3,13 @@
 from __future__ import division
 
 from sympy import (
-    integrate, Integral, exp, I, oo, pi, sign, sqrt, Rational, Symbol, sin, cos,
-    tan, S, log, Function, gamma, sinh,
+    integrate, Integral, exp, oo, pi, sign, sqrt, sin, cos,
+    tan, S, log, gamma, sinh,
 )
 
 from sympy.utilities.pytest import XFAIL, SKIP, slow, skip, ON_TRAVIS
 
-from sympy.abc import x, k, c, y, R, b, h, a, m, A, z, t
+from sympy.abc import x, k, c, y, R, b, h, a, m
 
 import signal
 
@@ -110,6 +110,7 @@ def test_issue_4891():
 
 
 @XFAIL
+@slow
 def test_issue_1796a():
     assert not integrate(exp(2*b*x)*exp(-a*x**2), x).has(Integral)
 
@@ -138,20 +139,6 @@ def test_issue_4941():
 
 
 @XFAIL
-@slow
-def test_issue_4950():
-    # Problem is with exception
-    assert not integrate((-60*exp(x) - 19.2*exp(4*x))*exp(4*x), x).has(Integral)
-
-
-@XFAIL
-@slow
-def test_issue_4968():
-    assert not integrate(sin(log(x**2))).has(Integral)
-
-
-@XFAIL
-@slow
 def test_issue_4992():
     # Nonelementary integral.  Requires hypergeometric/Meijer-G handling.
     assert not integrate(log(x) * x**(k - 1) * exp(-x) / gamma(k), (x, 0, oo)).has(Integral)
