@@ -13,7 +13,7 @@ def sympify_matrix(arg):
     return arg.as_immutable()
 sympify_converter[MatrixBase] = sympify_matrix
 
-class ImmutableMatrix(MatrixExpr, DenseMatrix):
+class ImmutableDenseMatrix(DenseMatrix, MatrixExpr):
     """Create an immutable version of a matrix.
 
     Examples
@@ -32,41 +32,52 @@ class ImmutableMatrix(MatrixExpr, DenseMatrix):
     TypeError: Cannot set values of ImmutableDenseMatrix
     """
 
+    # MatrixExpr is set as NotIterable, but we want explicit matrices to be
+    # iterable
+    _iterable = True
     _class_priority = 8
-
-    @classmethod
-    def _new(cls, *args, **kwargs):
-        if len(args) == 1 and isinstance(args[0], ImmutableMatrix):
-            return args[0]
-        rows, cols, flat_list = cls._handle_creation_inputs(*args, **kwargs)
-        rows = Integer(rows)
-        cols = Integer(cols)
-        mat = Tuple(*flat_list)
-        return Basic.__new__(cls, rows, cols, mat)
+    _op_priority = 10.001
 
     def __new__(cls, *args, **kwargs):
         return cls._new(*args, **kwargs)
 
-    @property
-    def shape(self):
-        return tuple([int(i) for i in self.args[:2]])
+    __hash__ = MatrixExpr.__hash__
+
+    @classmethod
+    def _new(cls, *args, **kwargs):
+        if len(args) == 1 and isinstance(args[0], ImmutableDenseMatrix):
+            return args[0]
+        if kwargs.get('copy', True) is False:
+            if len(args) != 3:
+                raise TypeError("'copy=False' requires a matrix be initialized as rows,cols,[list]")
+            rows, cols, flat_list = args
+        else:
+            rows, cols, flat_list = cls._handle_creation_inputs(*args, **kwargs)
+            flat_list = list(flat_list) # create a shallow copy
+        rows = Integer(rows)
+        cols = Integer(cols)
+        if not isinstance(flat_list, Tuple):
+            flat_list = Tuple(*flat_list)
+
+        return Basic.__new__(cls, rows, cols, flat_list)
 
     @property
     def _mat(self):
-        return list(self.args[2])
+        # self.args[2] is a Tuple.  Access to the elements
+        # of a tuple are significantly faster than Tuple,
+        # so return the internal tuple.
+        return self.args[2].args
 
     def _entry(self, i, j):
         return DenseMatrix.__getitem__(self, (i, j))
 
-    __getitem__ = DenseMatrix.__getitem__
-
     def __setitem__(self, *args):
-        raise TypeError("Cannot set values of ImmutableMatrix")
+        raise TypeError("Cannot set values of {}".format(self.__class__))
 
     def _eval_Eq(self, other):
         """Helper method for Equality with matrices.
 
-        Relational automatically converts matrices to ImmutableMatrix
+        Relational automatically converts matrices to ImmutableDenseMatrix
         instances, so this method only applies here.  Returns True if the
         matrices are definitively the same, False if they are definitively
         different, and None if undetermined (e.g. if they contain Symbols).
@@ -76,44 +87,43 @@ class ImmutableMatrix(MatrixExpr, DenseMatrix):
         if not hasattr(other, 'shape') or self.shape != other.shape:
             return S.false
         if isinstance(other, MatrixExpr) and not isinstance(
-                other, ImmutableMatrix):
+                other, ImmutableDenseMatrix):
             return None
         diff = self - other
         return sympify(diff.is_zero)
 
-    adjoint = MatrixBase.adjoint
-    conjugate = MatrixBase.conjugate
-    # C and T are defined in MatrixExpr...I don't know why C alone
-    # needs to be defined here
-    C = MatrixBase.C
+    def _eval_extract(self, rowsList, colsList):
+        # self._mat is a Tuple.  It is slightly faster to index a
+        # tuple over a Tuple, so grab the internal tuple directly
+        mat = self._mat
+        cols = self.cols
+        indices = (i * cols + j for i in rowsList for j in colsList)
+        return self._new(len(rowsList), len(colsList),
+                         Tuple(*(mat[i] for i in indices), sympify=False), copy=False)
 
-    as_mutable = DenseMatrix.as_mutable
-    _eval_trace = DenseMatrix._eval_trace
-    _eval_transpose = DenseMatrix._eval_transpose
-    _eval_conjugate = DenseMatrix._eval_conjugate
-    _eval_adjoint = DenseMatrix._eval_adjoint
-    _eval_inverse = DenseMatrix._eval_inverse
-    _eval_simplify = DenseMatrix._eval_simplify
+    @property
+    def cols(self):
+        return int(self.args[1])
 
-    equals = DenseMatrix.equals
-    is_Identity = DenseMatrix.is_Identity
+    @property
+    def rows(self):
+        return int(self.args[0])
+
+    @property
+    def shape(self):
+        return tuple(int(i) for i in self.args[:2])
 
-    __add__ = MatrixBase.__add__
-    __radd__ = MatrixBase.__radd__
-    __mul__ = MatrixBase.__mul__
-    __rmul__ = MatrixBase.__rmul__
-    __pow__ = MatrixBase.__pow__
-    __sub__ = MatrixBase.__sub__
-    __rsub__ = MatrixBase.__rsub__
-    __neg__ = MatrixBase.__neg__
-    __div__ = MatrixBase.__div__
-    __truediv__ = MatrixBase.__truediv__
 # This is included after the class definition as a workaround for issue 7213.
 # See https://github.com/sympy/sympy/issues/7213
-ImmutableMatrix.is_zero = DenseMatrix.is_zero
+# the object is non-zero
+# See https://github.com/sympy/sympy/issues/7213
+ImmutableDenseMatrix.is_zero = DenseMatrix.is_zero
+
+# make sure ImmutableDenseMatrix is aliased as ImmutableMatrix
+ImmutableMatrix = ImmutableDenseMatrix
 
 
-class ImmutableSparseMatrix(Basic, SparseMatrix):
+class ImmutableSparseMatrix(SparseMatrix, Basic):
     """Create an immutable version of a sparse matrix.
 
     Examples
@@ -135,7 +145,7 @@ class ImmutableSparseMatrix(Basic, SparseMatrix):
     >>> _.shape
     (3, 3)
     """
-
+    is_Matrix = True
     _class_priority = 9
 
     @classmethod
@@ -156,9 +166,7 @@ class ImmutableSparseMatrix(Basic, SparseMatrix):
     def __setitem__(self, *args):
         raise TypeError("Cannot set values of ImmutableSparseMatrix")
 
-    subs = MatrixBase.subs
-
     def __hash__(self):
         return hash((type(self).__name__,) + (self.shape, tuple(self._smat)))
 
-    _eval_Eq = ImmutableMatrix._eval_Eq
+    _eval_Eq = ImmutableDenseMatrix._eval_Eq
