@@ -1,9 +1,8 @@
 # -*- coding: utf-8 -*-
 
 from sympy import Derivative
-from sympy.core import C
-from sympy.core.compatibility import u
 from sympy.core.function import UndefinedFunction
+from sympy.core.symbol import Symbol
 from sympy.interactive.printing import init_printing
 from sympy.printing.conventions import split_super_sub
 from sympy.printing.latex import LatexPrinter, translate
@@ -132,7 +131,7 @@ class VectorLatexPrinter(LatexPrinter):
         expr = der_expr.expr
         red = expr.atoms(AppliedUndef)
         syms = der_expr.variables
-        test1 = not all([True for i in red if i.free_symbols == set([t])])
+        test1 = not all([True for i in red if i.free_symbols == {t}])
         test2 = not all([(t == i) for i in syms])
         if test1 or test2:
             return LatexPrinter().doprint(der_expr)
@@ -152,6 +151,13 @@ class VectorLatexPrinter(LatexPrinter):
             base += '_' + base_split[1]
         return base
 
+    def parenthesize(self, item, level, strict=False):
+        item_latex = self._print(item)
+        if item_latex.startswith(r"\dot") or item_latex.startswith(r"\ddot") or item_latex.startswith(r"\dddot"):
+            return self._print(item)
+        else:
+            return LatexPrinter.parenthesize(self, item, level, strict)
+
 
 class VectorPrettyPrinter(PrettyPrinter):
     """Pretty Printer for vectorialexpressions. """
@@ -182,11 +188,11 @@ class VectorPrettyPrinter(PrettyPrinter):
         if len(pform.picture) > 1:
             return super(VectorPrettyPrinter, self)._print_Derivative(deriv)
 
-        dots = {0 : u(""),
-                1 : u("\u0307"),
-                2 : u("\u0308"),
-                3 : u("\u20db"),
-                4 : u("\u20dc")}
+        dots = {0 : u"",
+                1 : u"\N{COMBINING DOT ABOVE}",
+                2 : u"\N{COMBINING DIAERESIS}",
+                3 : u"\N{COMBINING THREE DOTS ABOVE}",
+                4 : u"\N{COMBINING FOUR DOTS ABOVE}"}
 
         d = pform.__dict__
         pic = d['picture'][0]
@@ -208,7 +214,7 @@ class VectorPrettyPrinter(PrettyPrinter):
         func = e.func
         args = e.args
         func_name = func.__name__
-        pform = self._print_Symbol(C.Symbol(func_name))
+        pform = self._print_Symbol(Symbol(func_name))
         # If this function is an Undefined function of t, it is probably a
         # dynamic symbol, so we'll skip the (t). The rest of the code is
         # identical to the normal PrettyPrinter code
