@@ -11,6 +11,7 @@ from __future__ import print_function, division
 from sympy.polys.agca.modules import (Module, FreeModule, QuotientModule,
     SubModule, SubQuotientModule)
 from sympy.polys.polyerrors import CoercionFailed
+from sympy.core.compatibility import range
 
 # The main computational task for module homomorphisms is kernels.
 # For this reason, the concrete classes are organised by domain module type.
@@ -38,7 +39,7 @@ class ModuleHomomorphism(object):
     - domain - the domain module
     - codomain - the codomain module
     - _ker - cached kernel
-    - _img - cachd image
+    - _img - cached image
 
     Non-implemented methods:
 
@@ -419,7 +420,7 @@ class MatrixHomomorphism(ModuleHomomorphism):
     Helper class for all homomoprhisms which are expressed via a matrix.
 
     That is, for such homomorphisms ``domain`` is contained in a module
-    generated by finitely many elements `e_1, \dots, e_n`, so that the
+    generated by finitely many elements `e_1, \ldots, e_n`, so that the
     homomorphism is determined uniquely by its action on the `e_i`. It
     can thus be represented as a vector of elements of the codomain module,
     or potentially a supermodule of the codomain module
@@ -595,8 +596,8 @@ def homomorphism(domain, codomain, matrix):
     >>> R = QQ.old_poly_ring(x)
     >>> T = R.free_module(2)
 
-    If ``domain`` is a free module generated by `e_1, \dots, e_n`, then
-    ``matrix`` should be an n-element iterable `(b_1, \dots, b_n)` where
+    If ``domain`` is a free module generated by `e_1, \ldots, e_n`, then
+    ``matrix`` should be an n-element iterable `(b_1, \ldots, b_n)` where
     the `b_i` are elements of ``codomain``. The constructed homomorphism is the
     unique homomorphism sending `e_i` to `b_i`.
 
