@@ -26,7 +26,7 @@ from sympy.polys.orderings import ProductOrder, monomial_key
 from sympy.polys.domains.field import Field
 from sympy.polys.agca.ideals import Ideal
 
-from sympy.core.compatibility import iterable, reduce
+from sympy.core.compatibility import iterable, reduce, range
 
 # TODO
 # - module saturation
@@ -795,9 +795,9 @@ class SubModule(Module):
         r"""
         Compute the syzygy module of the generators of ``self``.
 
-        Suppose `M` is generated by `f_1, \dots, f_n` over the ring
+        Suppose `M` is generated by `f_1, \ldots, f_n` over the ring
         `R`. Consider the homomorphism `\phi: R^n \to M`, given by
-        sending `(r_1, \dots, r_n) \to r_1 f_1 + \dots + r_n f_n`.
+        sending `(r_1, \ldots, r_n) \to r_1 f_1 + \cdots + r_n f_n`.
         The syzygy module is defined to be the kernel of `\phi`.
 
         The syzygy module is zero iff the generators generate freely a free
@@ -1245,7 +1245,6 @@ class QuotientModuleElement(ModuleElement):
         return self.module.killed_module.contains(d1 - d2)
 
     def __repr__(self):
-        from sympy import sstr
         return repr(self.data) + " + " + repr(self.module.killed_module)
 
 
