@@ -6,7 +6,7 @@ import sys
 import warnings
 unicode_warnings = ''
 
-from sympy.core.compatibility import u, unicode, range
+from sympy.core.compatibility import unicode, range
 
 # first, setup unicodedate environment
 try:
@@ -120,14 +120,14 @@ greek_letters = list(greeks) # make a copy
 greek_letters[greek_letters.index('lambda')] = 'lamda'
 
 # {}  greek letter -> (g,G)
-greek_unicode = dict([(l, (g(l), G(l))) for l in greek_letters])
+greek_unicode = {l: (g(l), G(l)) for l in greek_letters}
 greek_unicode = dict((L, g(L)) for L in greek_letters)
 greek_unicode.update((L[0].upper() + L[1:], G(L)) for L in greek_letters)
 
 # aliases
 greek_unicode['lambda'] = greek_unicode['lamda']
 greek_unicode['Lambda'] = greek_unicode['Lamda']
-greek_unicode['varsigma'] = u('\N{GREEK SMALL LETTER FINAL SIGMA}')
+greek_unicode['varsigma'] = u'\N{GREEK SMALL LETTER FINAL SIGMA}'
 
 digit_2txt = {
     '0':    'ZERO',
@@ -195,21 +195,21 @@ for s in '+-=()':
 # TODO: Make brackets adjust to height of contents
 modifier_dict = {
     # Accents
-    'mathring': lambda s: s+u('\N{COMBINING RING ABOVE}'),
-    'ddddot': lambda s: s+u('\N{COMBINING DIAERESIS}\N{COMBINING DIAERESIS}'),
-    'dddot': lambda s: s+u('\N{COMBINING DIAERESIS}\N{COMBINING DOT ABOVE}'),
-    'ddot': lambda s: s+u('\N{COMBINING DIAERESIS}'),
-    'dot': lambda s: s+u('\N{COMBINING DOT ABOVE}'),
-    'check': lambda s: s+u('\N{COMBINING CARON}'),
-    'breve': lambda s: s+u('\N{COMBINING BREVE}'),
-    'acute': lambda s: s+u('\N{COMBINING ACUTE ACCENT}'),
-    'grave': lambda s: s+u('\N{COMBINING GRAVE ACCENT}'),
-    'tilde': lambda s: s+u('\N{COMBINING TILDE}'),
-    'hat': lambda s: s+u('\N{COMBINING CIRCUMFLEX ACCENT}'),
-    'bar': lambda s: s+u('\N{COMBINING OVERLINE}'),
-    'vec': lambda s: s+u('\N{COMBINING RIGHT ARROW ABOVE}'),
-    'prime': lambda s: s+u('\N{PRIME}'),
-    'prm': lambda s: s+u('\N{PRIME}'),
+    'mathring': lambda s: s+u'\N{COMBINING RING ABOVE}',
+    'ddddot': lambda s: s+u'\N{COMBINING DIAERESIS}\N{COMBINING DIAERESIS}',
+    'dddot': lambda s: s+u'\N{COMBINING DIAERESIS}\N{COMBINING DOT ABOVE}',
+    'ddot': lambda s: s+u'\N{COMBINING DIAERESIS}',
+    'dot': lambda s: s+u'\N{COMBINING DOT ABOVE}',
+    'check': lambda s: s+u'\N{COMBINING CARON}',
+    'breve': lambda s: s+u'\N{COMBINING BREVE}',
+    'acute': lambda s: s+u'\N{COMBINING ACUTE ACCENT}',
+    'grave': lambda s: s+u'\N{COMBINING GRAVE ACCENT}',
+    'tilde': lambda s: s+u'\N{COMBINING TILDE}',
+    'hat': lambda s: s+u'\N{COMBINING CIRCUMFLEX ACCENT}',
+    'bar': lambda s: s+u'\N{COMBINING OVERLINE}',
+    'vec': lambda s: s+u'\N{COMBINING RIGHT ARROW ABOVE}',
+    'prime': lambda s: s+u'\N{PRIME}',
+    'prm': lambda s: s+u'\N{PRIME}',
     # # Faces -- these are here for some compatibility with latex printing
     # 'bold': lambda s: s,
     # 'bm': lambda s: s,
@@ -217,10 +217,10 @@ modifier_dict = {
     # 'scr': lambda s: s,
     # 'frak': lambda s: s,
     # Brackets
-    'norm': lambda s: u('\N{DOUBLE VERTICAL LINE}')+s+u('\N{DOUBLE VERTICAL LINE}'),
-    'avg': lambda s: u('\N{MATHEMATICAL LEFT ANGLE BRACKET}')+s+u('\N{MATHEMATICAL RIGHT ANGLE BRACKET}'),
-    'abs': lambda s: u('\N{VERTICAL LINE}')+s+u('\N{VERTICAL LINE}'),
-    'mag': lambda s: u('\N{VERTICAL LINE}')+s+u('\N{VERTICAL LINE}'),
+    'norm': lambda s: u'\N{DOUBLE VERTICAL LINE}'+s+u'\N{DOUBLE VERTICAL LINE}',
+    'avg': lambda s: u'\N{MATHEMATICAL LEFT ANGLE BRACKET}'+s+u'\N{MATHEMATICAL RIGHT ANGLE BRACKET}',
+    'abs': lambda s: u'\N{VERTICAL LINE}'+s+u'\N{VERTICAL LINE}',
+    'mag': lambda s: u'\N{VERTICAL LINE}'+s+u'\N{VERTICAL LINE}',
 }
 
 # VERTICAL OBJECTS
@@ -432,6 +432,11 @@ _xsym = {
     '>=':  ('>=', U('GREATER-THAN OR EQUAL TO')),
     '!=':  ('!=', U('NOT EQUAL TO')),
     ':=':  (':=', ':='),
+    '+=':  ('+=', '+='),
+    '-=':  ('-=', '-='),
+    '*=':  ('*=', '*='),
+    '/=':  ('/=', '/='),
+    '%=':  ('%=', '%='),
     '*':   ('*', U('DOT OPERATOR')),
     '-->': ('-->', U('EM DASH') + U('EM DASH') +
             U('BLACK RIGHT-POINTING TRIANGLE') if U('EM DASH')
@@ -559,17 +564,16 @@ def annotated(letter):
     information.
     """
     ucode_pics = {
-        'F': (2, 0, 2, 0, u('\N{BOX DRAWINGS LIGHT DOWN AND RIGHT}\N{BOX DRAWINGS LIGHT HORIZONTAL}\n'
-                            '\N{BOX DRAWINGS LIGHT VERTICAL AND RIGHT}\N{BOX DRAWINGS LIGHT HORIZONTAL}\n'
-                            '\N{BOX DRAWINGS LIGHT UP}')),
-        'G': (3, 0, 3, 1,
-              u('\N{BOX DRAWINGS LIGHT ARC DOWN AND RIGHT}\N{BOX DRAWINGS LIGHT HORIZONTAL}\N{BOX DRAWINGS LIGHT ARC DOWN AND LEFT}\n'
-                '\N{BOX DRAWINGS LIGHT VERTICAL}\N{BOX DRAWINGS LIGHT RIGHT}\N{BOX DRAWINGS LIGHT DOWN AND LEFT}\n'
-                '\N{BOX DRAWINGS LIGHT ARC UP AND RIGHT}\N{BOX DRAWINGS LIGHT HORIZONTAL}\N{BOX DRAWINGS LIGHT ARC UP AND LEFT}'))
+        'F': (2, 0, 2, 0, u'\N{BOX DRAWINGS LIGHT DOWN AND RIGHT}\N{BOX DRAWINGS LIGHT HORIZONTAL}\n'
+                          u'\N{BOX DRAWINGS LIGHT VERTICAL AND RIGHT}\N{BOX DRAWINGS LIGHT HORIZONTAL}\n'
+                          u'\N{BOX DRAWINGS LIGHT UP}'),
+        'G': (3, 0, 3, 1, u'\N{BOX DRAWINGS LIGHT ARC DOWN AND RIGHT}\N{BOX DRAWINGS LIGHT HORIZONTAL}\N{BOX DRAWINGS LIGHT ARC DOWN AND LEFT}\n'
+                          u'\N{BOX DRAWINGS LIGHT VERTICAL}\N{BOX DRAWINGS LIGHT RIGHT}\N{BOX DRAWINGS LIGHT DOWN AND LEFT}\n'
+                          u'\N{BOX DRAWINGS LIGHT ARC UP AND RIGHT}\N{BOX DRAWINGS LIGHT HORIZONTAL}\N{BOX DRAWINGS LIGHT ARC UP AND LEFT}')
     }
     ascii_pics = {
         'F': (3, 0, 3, 0, ' _\n|_\n|\n'),
-        'G': (3, 0, 3, 1, ' __\n/__\n\_|')
+        'G': (3, 0, 3, 1, ' __\n/__\n\\_|')
     }
 
     if _use_unicode:
