@@ -1,7 +1,6 @@
 from sympy import diff, Integral, Limit, sin, Symbol, Integer, Rational, cos, \
     tan, asin, acos, atan, sinh, cosh, tanh, asinh, acosh, atanh, E, I, oo, \
-    pi, GoldenRatio, EulerGamma, Sum, Eq, Ne, Ge, Lt, Float
-from sympy.core.compatibility import u
+    pi, GoldenRatio, EulerGamma, Sum, Eq, Ne, Ge, Lt, Float, Matrix
 from sympy.printing.mathml import mathml, MathMLPrinter
 
 from sympy.utilities.pytest import raises
@@ -10,7 +9,6 @@ x = Symbol('x')
 y = Symbol('y')
 mp = MathMLPrinter()
 
-
 def test_printmethod():
     assert mp.doprint(1 + x) == '<apply><plus/><ci>x</ci><cn>1</cn></apply>'
 
@@ -92,6 +90,41 @@ def test_mathml_integrals():
     assert mml_1.childNodes[3].nodeName == 'uplimit'
     assert mml_1.childNodes[4].toxml() == mp._print(integrand).toxml()
 
+def test_mathml_matrices():
+    A = Matrix([1, 2, 3])
+    B = Matrix([[0, 5, 4], [2, 3, 1], [9, 7, 9]])
+    mll_1 = mp._print(A)
+    assert mll_1.childNodes[0].nodeName == 'matrixrow'
+    assert mll_1.childNodes[0].childNodes[0].nodeName == 'cn'
+    assert mll_1.childNodes[0].childNodes[0].childNodes[0].nodeValue == '1'
+    assert mll_1.childNodes[1].nodeName == 'matrixrow'
+    assert mll_1.childNodes[1].childNodes[0].nodeName == 'cn'
+    assert mll_1.childNodes[1].childNodes[0].childNodes[0].nodeValue == '2'
+    assert mll_1.childNodes[2].nodeName == 'matrixrow'
+    assert mll_1.childNodes[2].childNodes[0].nodeName == 'cn'
+    assert mll_1.childNodes[2].childNodes[0].childNodes[0].nodeValue == '3'
+    mll_2 = mp._print(B)
+    assert mll_2.childNodes[0].nodeName == 'matrixrow'
+    assert mll_2.childNodes[0].childNodes[0].nodeName == 'cn'
+    assert mll_2.childNodes[0].childNodes[0].childNodes[0].nodeValue == '0'
+    assert mll_2.childNodes[0].childNodes[1].nodeName == 'cn'
+    assert mll_2.childNodes[0].childNodes[1].childNodes[0].nodeValue == '5'
+    assert mll_2.childNodes[0].childNodes[2].nodeName == 'cn'
+    assert mll_2.childNodes[0].childNodes[2].childNodes[0].nodeValue == '4'
+    assert mll_2.childNodes[1].nodeName == 'matrixrow'
+    assert mll_2.childNodes[1].childNodes[0].nodeName == 'cn'
+    assert mll_2.childNodes[1].childNodes[0].childNodes[0].nodeValue == '2'
+    assert mll_2.childNodes[1].childNodes[1].nodeName == 'cn'
+    assert mll_2.childNodes[1].childNodes[1].childNodes[0].nodeValue == '3'
+    assert mll_2.childNodes[1].childNodes[2].nodeName == 'cn'
+    assert mll_2.childNodes[1].childNodes[2].childNodes[0].nodeValue == '1'
+    assert mll_2.childNodes[2].nodeName == 'matrixrow'
+    assert mll_2.childNodes[2].childNodes[0].nodeName == 'cn'
+    assert mll_2.childNodes[2].childNodes[0].childNodes[0].nodeValue == '9'
+    assert mll_2.childNodes[2].childNodes[1].nodeName == 'cn'
+    assert mll_2.childNodes[2].childNodes[1].childNodes[0].nodeValue == '7'
+    assert mll_2.childNodes[2].childNodes[2].nodeName == 'cn'
+    assert mll_2.childNodes[2].childNodes[2].childNodes[0].nodeValue == '9'
 
 def test_mathml_sums():
     summand = x
@@ -333,7 +366,7 @@ def test_symbol():
 def test_mathml_greek():
     mml = mp._print(Symbol('alpha'))
     assert mml.nodeName == 'ci'
-    assert mml.childNodes[0].nodeValue == u('\N{GREEK SMALL LETTER ALPHA}')
+    assert mml.childNodes[0].nodeValue == u'\N{GREEK SMALL LETTER ALPHA}'
 
     assert mp.doprint(Symbol('alpha')) == '<ci>&#945;</ci>'
     assert mp.doprint(Symbol('beta')) == '<ci>&#946;</ci>'
