@@ -14,9 +14,10 @@ from sympy.stats.rv import (RandomDomain, SingleDomain, ConditionalDomain,
         ProductDomain, PSpace, SinglePSpace, random_symbols, ProductPSpace,
         NamedArgsMixin)
 from sympy.functions.special.delta_functions import DiracDelta
-from sympy import (S, Interval, symbols, sympify, Dummy, FiniteSet, Mul, Tuple,
-        Integral, And, Or, Piecewise, solve, cacheit, integrate, oo, Lambda,
-        Basic)
+from sympy import (Interval, Intersection, symbols, sympify, Dummy, Mul,
+        Integral, And, Or, Piecewise, cacheit, integrate, oo, Lambda,
+        Basic, S)
+from sympy.solvers.solveset import solveset
 from sympy.solvers.inequalities import reduce_rational_inequalities
 from sympy.polys.polyerrors import PolynomialError
 import random
@@ -181,7 +182,9 @@ class SingleContinuousDistribution(ContinuousDistribution, NamedArgsMixin):
         x, z = symbols('x, z', real=True, positive=True, cls=Dummy)
         # Invert CDF
         try:
-            inverse_cdf = solve(self.cdf(x) - z, x)
+            inverse_cdf = solveset(self.cdf(x) - z, x, S.Reals)
+            if isinstance(inverse_cdf, Intersection) and S.Reals in inverse_cdf.args:
+                inverse_cdf = list(inverse_cdf.args[1])
         except NotImplementedError:
             inverse_cdf = None
         if not inverse_cdf or len(inverse_cdf) != 1:
@@ -297,6 +300,9 @@ class ContinuousPSpace(PSpace):
             rv = [rv for rv in self.values if rv.symbol == domain.symbol][0]
             # Integrate out all other random variables
             pdf = self.compute_density(rv, **kwargs)
+            # return S.Zero if `domain` is empty set
+            if domain.set is S.EmptySet:
+                return S.Zero
             # Integrate out the last variable over the special domain
             return Integral(pdf(z), (z, domain.set), **kwargs)
 
@@ -384,7 +390,12 @@ class SingleContinuousPSpace(ContinuousPSpace, SinglePSpace):
         if expr == self.value:
             return self.density
         y = Dummy('y')
-        gs = solve(expr - y, self.value)
+
+        gs = solveset(expr - y, self.value, S.Reals)
+
+        if isinstance(gs, Intersection) and S.Reals in gs.args:
+            gs = list(gs.args[1])
+
         if not gs:
             raise ValueError("Can not solve %s for %s"%(expr, self.value))
         fx = self.compute_density(self.value)
