@@ -5,16 +5,21 @@ lambda functions which can be used to calculate numerical values very fast.
 
 from __future__ import print_function, division
 
-from sympy.external import import_module
-from sympy.core.compatibility import exec_, is_sequence, iterable, string_types
-
+from functools import wraps
 import inspect
+import textwrap
+
+from sympy.core.compatibility import (exec_, is_sequence, iterable,
+    NotIterable, string_types, range, builtins, integer_types)
+from sympy.utilities.decorator import doctest_depends_on
 
 # These are the namespaces the lambda functions will use.
 MATH = {}
 MPMATH = {}
 NUMPY = {}
+TENSORFLOW = {}
 SYMPY = {}
+NUMEXPR = {}
 
 # Default namespaces, letting us define translations that can't be defined
 # by simple variable maps, like I => 1j
@@ -23,11 +28,12 @@ SYMPY = {}
 MATH_DEFAULT = {}
 MPMATH_DEFAULT = {}
 NUMPY_DEFAULT = {"I": 1j}
+TENSORFLOW_DEFAULT = {}
 SYMPY_DEFAULT = {}
+NUMEXPR_DEFAULT = {}
 
 # Mappings between sympy and other modules function names.
 MATH_TRANSLATIONS = {
-    "Abs": "fabs",
     "ceiling": "ceil",
     "E": "e",
     "ln": "log",
@@ -49,9 +55,8 @@ MPMATH_TRANSLATIONS = {
     "oo": "inf",
     #"uppergamma":"upper_gamma",
     "LambertW": "lambertw",
-    "Matrix": "matrix",
     "MutableDenseMatrix": "matrix",
-    "ImmutableMatrix": "matrix",
+    "ImmutableDenseMatrix": "matrix",
     "conjugate": "conj",
     "dirichlet_eta": "altzeta",
     "Ei": "ei",
@@ -62,7 +67,6 @@ MPMATH_TRANSLATIONS = {
 }
 
 NUMPY_TRANSLATIONS = {
-    "Abs": "abs",
     "acos": "arccos",
     "acosh": "arccosh",
     "arg": "angle",
@@ -75,24 +79,40 @@ NUMPY_TRANSLATIONS = {
     "E": "e",
     "im": "imag",
     "ln": "log",
-    "Matrix": "matrix",
-    "MutableDenseMatrix": "matrix",
-    "ImmutableMatrix": "matrix",
-    "Max": "amax",
-    "Min": "amin",
+    "Mod": "mod",
     "oo": "inf",
     "re": "real",
+    "SparseMatrix": "array",
+    "ImmutableSparseMatrix": "array",
+    "Matrix": "array",
+    "MutableDenseMatrix": "array",
+    "ImmutableDenseMatrix": "array",
 }
 
+TENSORFLOW_TRANSLATIONS = {
+    "Abs": "abs",
+    "ceiling": "ceil",
+    "im": "imag",
+    "ln": "log",
+    "Mod": "mod",
+    "conjugate": "conj",
+    "re": "real",
+}
+
+NUMEXPR_TRANSLATIONS = {}
+
 # Available modules:
 MODULES = {
     "math": (MATH, MATH_DEFAULT, MATH_TRANSLATIONS, ("from math import *",)),
-    "mpmath": (MPMATH, MPMATH_DEFAULT, MPMATH_TRANSLATIONS, ("from sympy.mpmath import *",)),
+    "mpmath": (MPMATH, MPMATH_DEFAULT, MPMATH_TRANSLATIONS, ("from mpmath import *",)),
     "numpy": (NUMPY, NUMPY_DEFAULT, NUMPY_TRANSLATIONS, ("import_module('numpy')",)),
+    "tensorflow": (TENSORFLOW, TENSORFLOW_DEFAULT, TENSORFLOW_TRANSLATIONS, ("import_module('tensorflow')",)),
     "sympy": (SYMPY, SYMPY_DEFAULT, {}, (
         "from sympy.functions import *",
         "from sympy.matrices import *",
         "from sympy import Integral, pi, oo, nan, zoo, E, I",)),
+    "numexpr" : (NUMEXPR, NUMEXPR_DEFAULT, NUMEXPR_TRANSLATIONS,
+                 ("import_module('numexpr')", )),
 }
 
 
@@ -101,10 +121,11 @@ def _import(module, reload="False"):
     Creates a global translation dictionary for module.
 
     The argument module has to be one of the following strings: "math",
-    "mpmath", "numpy", "sympy".
+    "mpmath", "numpy", "sympy", "tensorflow".
     These dictionaries map names of python functions to their equivalent in
     other modules.
     """
+    from sympy.external import import_module
     try:
         namespace, namespace_default, translations, import_commands = MODULES[
             module]
@@ -142,22 +163,40 @@ def _import(module, reload="False"):
     for sympyname, translation in translations.items():
         namespace[sympyname] = namespace[translation]
 
+    # For computing the modulus of a sympy expression we use the builtin abs
+    # function, instead of the previously used fabs function for all
+    # translation modules. This is because the fabs function in the math
+    # module does not accept complex valued arguments. (see issue 9474). The
+    # only exception, where we don't use the builtin abs function is the
+    # mpmath translation module, because mpmath.fabs returns mpf objects in
+    # contrast to abs().
+    if 'Abs' not in namespace:
+        namespace['Abs'] = abs
 
-def lambdify(args, expr, modules=None, printer=None, use_imps=True, dummify=True):
+
+@doctest_depends_on(modules=('numpy'))
+def lambdify(args, expr, modules=None, printer=None, use_imps=True,
+             dummify=True):
     """
     Returns a lambda function for fast calculation of numerical values.
 
-    If not specified differently by the user, SymPy functions are replaced as
-    far as possible by either python-math, numpy (if available) or mpmath
-    functions - exactly in this order. To change this behavior, the "modules"
-    argument can be used. It accepts:
+    If not specified differently by the user, ``modules`` defaults to
+    ``["numpy"]`` if NumPy is installed, and ``["math", "mpmath", "sympy"]``
+    if it isn't, that is, SymPy functions are replaced as far as possible by
+    either ``numpy`` functions if available, and Python's standard library
+    ``math``, or ``mpmath`` functions otherwise. To change this behavior, the
+    "modules" argument can be used. It accepts:
 
-     - the strings "math", "mpmath", "numpy", "sympy"
+     - the strings "math", "mpmath", "numpy", "numexpr", "sympy", "tensorflow"
      - any modules (e.g. math)
      - dictionaries that map names of sympy functions to arbitrary functions
      - lists that contain a mix of the arguments above, with higher priority
        given to entries appearing first.
 
+    .. warning::
+        Note that this function uses ``eval``, and thus shouldn't be used on
+        unsanitized input.
+
     The default behavior is to substitute all arguments in the provided
     expression with dummy symbols. This allows for applied functions (e.g.
     f(t)) to be supplied as arguments. Call the function with dummify=False if
@@ -165,14 +204,36 @@ def lambdify(args, expr, modules=None, printer=None, use_imps=True, dummify=True
     to view the lambdified function or provide "sympy" as the module, you
     should probably set dummify=False.
 
+    For functions involving large array calculations, numexpr can provide a
+    significant speedup over numpy.  Please note that the available functions
+    for numexpr are more limited than numpy but can be expanded with
+    implemented_function and user defined subclasses of Function.  If specified,
+    numexpr may be the only option in modules. The official list of numexpr
+    functions can be found at:
+    https://github.com/pydata/numexpr#supported-functions
+
+    In previous releases ``lambdify`` replaced ``Matrix`` with ``numpy.matrix``
+    by default. As of release 1.0 ``numpy.array`` is the default.
+    To get the old default behavior you must pass in ``[{'ImmutableDenseMatrix':
+    numpy.matrix}, 'numpy']`` to the ``modules`` kwarg.
+
+    >>> from sympy import lambdify, Matrix
+    >>> from sympy.abc import x, y
+    >>> import numpy
+    >>> array2mat = [{'ImmutableDenseMatrix': numpy.matrix}, 'numpy']
+    >>> f = lambdify((x, y), Matrix([x, y]), modules=array2mat)
+    >>> f(1, 2)
+    matrix([[1],
+            [2]])
+
     Usage
     =====
 
     (1) Use one of the provided modules:
 
-        >>> from sympy import lambdify, sin, gamma
+        >>> from sympy import sin, tan, gamma
         >>> from sympy.utilities.lambdify import lambdastr
-        >>> from sympy.abc import x
+        >>> from sympy.abc import x, y
         >>> f = lambdify(x, sin(x), "math")
 
         Attention: Functions that are not in the math module will throw a name
@@ -183,19 +244,19 @@ def lambdify(args, expr, modules=None, printer=None, use_imps=True, dummify=True
 
     (2) Use some other module:
 
-        >> import numpy
-        >> f = lambdify((x,y), tan(x*y), numpy)
+        >>> import numpy
+        >>> f = lambdify((x,y), tan(x*y), numpy)
 
         Attention: There are naming differences between numpy and sympy. So if
                    you simply take the numpy module, e.g. sympy.atan will not be
                    translated to numpy.arctan. Use the modified module instead
                    by passing the string "numpy":
 
-        >> f = lambdify((x,y), tan(x*y), "numpy")
-        >> f(1, 2)
+        >>> f = lambdify((x,y), tan(x*y), "numpy")
+        >>> f(1, 2)
         -2.18503986326
-        >> from numpy import array
-        >> f(array([1, 2, 3]), array([2, 3, 5]))
+        >>> from numpy import array
+        >>> f(array([1, 2, 3]), array([2, 3, 5]))
         [-2.18503986 -0.29100619 -0.8559934 ]
 
     (3) Use a dictionary defining custom functions:
@@ -208,7 +269,7 @@ def lambdify(args, expr, modules=None, printer=None, use_imps=True, dummify=True
     Examples
     ========
 
-    >>> from sympy.utilities.lambdify import implemented_function, lambdify
+    >>> from sympy.utilities.lambdify import implemented_function
     >>> from sympy import sqrt, sin, Matrix
     >>> from sympy import Function
     >>> from sympy.abc import w, x, y, z
@@ -259,6 +320,27 @@ def lambdify(args, expr, modules=None, printer=None, use_imps=True, dummify=True
 
     ``lambdify`` always prefers ``_imp_`` implementations to implementations
     in other namespaces, unless the ``use_imps`` input parameter is False.
+
+    Usage with Tensorflow module:
+
+    >>> import tensorflow as tf
+    >>> f = Max(x, sin(x))
+    >>> func = lambdify(x, f, 'tensorflow')
+    >>> result = func(tf.constant(1.0))
+    >>> result # a tf.Tensor representing the result of the calculation
+    <tf.Tensor 'Maximum:0' shape=() dtype=float32>
+    >>> sess = tf.Session()
+    >>> sess.run(result) # compute result
+    1.0
+    >>> var = tf.Variable(1.0)
+    >>> sess.run(tf.global_variables_initializer())
+    >>> sess.run(func(var)) # also works for tf.Variable and tf.Placeholder
+    1.0
+    >>> tensor = tf.constant([[1.0, 2.0], [3.0, 4.0]]) # works with any shape tensor
+    >>> sess.run(func(tensor))
+    array([[ 1.,  2.],
+           [ 3.,  4.]], dtype=float32)
+
     """
     from sympy.core.symbol import Symbol
     from sympy.utilities.iterables import flatten
@@ -267,18 +349,16 @@ def lambdify(args, expr, modules=None, printer=None, use_imps=True, dummify=True
     module_provided = True
     if modules is None:
         module_provided = False
-        # Use either numpy (if available) or python.math where possible.
-        # XXX: This leads to different behaviour on different systems and
-        #      might be the reason for irreproducible errors.
-        modules = ["math", "mpmath", "sympy"]
 
         try:
             _import("numpy")
         except ImportError:
-            pass
+            # Use either numpy (if available) or python.math where possible.
+            # XXX: This leads to different behaviour on different systems and
+            #      might be the reason for irreproducible errors.
+            modules = ["math", "mpmath", "sympy"]
         else:
-            modules.insert(1, "numpy")
-
+            modules = ["numpy"]
 
     # Get the needed namespaces.
     namespaces = []
@@ -289,6 +369,9 @@ def lambdify(args, expr, modules=None, printer=None, use_imps=True, dummify=True
     if isinstance(modules, (dict, str)) or not hasattr(modules, '__iter__'):
         namespaces.append(modules)
     else:
+        # consistency check
+        if _module_present('numexpr', modules) and len(modules) > 1:
+            raise TypeError("numexpr must be the only item in 'modules'")
         namespaces += list(modules)
     # fill namespace with first having highest priority
     namespace = {}
@@ -303,13 +386,81 @@ def lambdify(args, expr, modules=None, printer=None, use_imps=True, dummify=True
         for term in syms:
             namespace.update({str(term): term})
 
+    if _module_present('mpmath',namespaces) and printer is None:
+        #XXX: This has to be done here because of circular imports
+        from sympy.printing.lambdarepr import MpmathPrinter as printer
+
+    if _module_present('numpy',namespaces) and printer is None:
+        #XXX: This has to be done here because of circular imports
+        from sympy.printing.lambdarepr import NumPyPrinter as printer
+
+    if _module_present('numexpr',namespaces) and printer is None:
+        #XXX: This has to be done here because of circular imports
+        from sympy.printing.lambdarepr import NumExprPrinter as printer
+
+    if _module_present('tensorflow',namespaces) and printer is None:
+        #XXX: This has to be done here because of circular imports
+        from sympy.printing.lambdarepr import TensorflowPrinter as printer
+
+    # Get the names of the args, for creating a docstring
+    if not iterable(args):
+        args = (args,)
+    names = []
+    # Grab the callers frame, for getting the names by inspection (if needed)
+    callers_local_vars = inspect.currentframe().f_back.f_locals.items()
+    for n, var in enumerate(args):
+        if hasattr(var, 'name'):
+            names.append(var.name)
+        else:
+            # It's an iterable. Try to get name by inspection of calling frame.
+            name_list = [var_name for var_name, var_val in callers_local_vars
+                    if var_val is var]
+            if len(name_list) == 1:
+                names.append(name_list[0])
+            else:
+                # Cannot infer name with certainty. arg_# will have to do.
+                names.append('arg_' + str(n))
+
     # Create lambda function.
     lstr = lambdastr(args, expr, printer=printer, dummify=dummify)
     flat = '__flatten_args__'
+
     if flat in lstr:
-        import itertools
         namespace.update({flat: flatten})
-    return eval(lstr, namespace)
+
+    # Provide lambda expression with builtins, and compatible implementation of range
+    namespace.update({'builtins':builtins, 'range':range})
+
+    func = eval(lstr, namespace)
+    # For numpy lambdify, wrap all input arguments in arrays.
+    # This is a fix for gh-11306.
+    if module_provided and _module_present('numpy',namespaces):
+        def array_wrap(funcarg):
+            @wraps(funcarg)
+            def wrapper(*argsx, **kwargsx):
+                asarray = namespace['asarray']
+                newargs = [asarray(i) if isinstance(i, integer_types + (float,
+                    complex)) else i for i in argsx]
+                return funcarg(*newargs, **kwargsx)
+            return wrapper
+        func = array_wrap(func)
+    # Apply the docstring
+    sig = "func({0})".format(", ".join(str(i) for i in names))
+    sig = textwrap.fill(sig, subsequent_indent=' '*8)
+    expr_str = str(expr)
+    if len(expr_str) > 78:
+        expr_str = textwrap.wrap(expr_str, 75)[0] + '...'
+    func.__doc__ = ("Created with lambdify. Signature:\n\n{sig}\n\n"
+                    "Expression:\n\n{expr}").format(sig=sig, expr=expr_str)
+    return func
+
+def _module_present(modname, modlist):
+    if modname in modlist:
+        return True
+    for m in modlist:
+        if hasattr(m, '__name__') and m.__name__ == modname:
+            return True
+    return False
 
 
 def _get_namespace(m):
@@ -373,7 +524,8 @@ def lambdastr(args, expr, printer=None, dummify=False):
             dummies = flatten([sub_args(a, dummies_dict) for a in args])
             return ",".join(str(a) for a in dummies)
         else:
-            if isinstance(args, Function):
+            #Sub in dummy variables for functions or symbols
+            if isinstance(args, (Function, Symbol)):
                 dummies = Dummy()
                 dummies_dict.update({args : dummies})
                 return str(dummies)
@@ -383,7 +535,7 @@ def lambdastr(args, expr, printer=None, dummify=False):
     def sub_expr(expr, dummies_dict):
         try:
             expr = sympify(expr).xreplace(dummies_dict)
-        except:
+        except Exception:
             if isinstance(expr, DeferredVector):
                 pass
             elif isinstance(expr, dict):
@@ -398,7 +550,8 @@ def lambdastr(args, expr, printer=None, dummify=False):
 
     # Transform args
     def isiter(l):
-        return iterable(l, exclude=(str, DeferredVector))
+        return iterable(l, exclude=(str, DeferredVector, NotIterable))
+
     if isiter(args) and any(isiter(i) for i in args):
         from sympy.utilities.iterables import flatten
         import re
@@ -457,7 +610,8 @@ def _imp_namespace(expr, namespace=None):
        function
 
     Examples
-    --------
+    ========
+
     >>> from sympy.abc import x
     >>> from sympy.utilities.lambdify import implemented_function, _imp_namespace
     >>> from sympy import Function
@@ -509,7 +663,7 @@ def implemented_function(symfunc, implementation):
 
     Be aware that this is a quick workaround, not a general method to create
     special symbolic functions. If you want to create a symbolic function to be
-    used by all the machinery of sympy you should subclass the ``Function``
+    used by all the machinery of SymPy you should subclass the ``Function``
     class.
 
     Parameters
@@ -526,7 +680,8 @@ def implemented_function(symfunc, implementation):
        function with attached implementation
 
     Examples
-    --------
+    ========
+
     >>> from sympy.abc import x
     >>> from sympy.utilities.lambdify import lambdify, implemented_function
     >>> from sympy import Function
